import datetime
import typing as T
from math import ceil
from pathlib import Path

import matplotlib.pyplot as plt

from my_weekly_schedule.models import Event, WeekdayEnum


def plot_events(
    events: list[Event],
    show: bool,
    with_weekends: bool,
    save_img: bool,
    out_path: T.Optional[Path] = None,
):
    """
    Generate the schedule as a matplotlib plot.
    """
    if save_img and out_path is None:
        raise ValueError("Attempted to save image, but no output path was given.")

    fig = plt.figure(figsize=(18, 9))

    for event in events:
        min_x = event.day.value + 0.5
        max_x = min_x + 0.96

        min_y = time_to_hours(event.start)
        max_y = time_to_hours(event.end)

        plt.fill_between(
            [min_x, max_x], [min_y, min_y], [max_y, max_y], color=event.color.as_named()
        )
        plt.text(
            min_x + 0.02,
            min_y + 0.02,
            event.start.strftime("%H:%M"),
            va="top",
            fontsize=8,
        )

        plt.text(
            (min_x + max_x) / 2,
            (min_y + max_y) / 2,
            event.title,
            ha="center",
            va="center",
            fontsize=10,
        )

    plt.title("Weekly Schedule", y=1, fontsize=14)

    [ax] = fig.axes

    days = [day.name.title() for day in WeekdayEnum if with_weekends or day.value < 5]

    ax.set_xlim(0.5, len(days) + 0.5)
    ax.set_xticks(range(1, len(days) + 1))
    ax.set_xticklabels(days)

    earliest = min(time_to_hours(event.start) for event in events)
    latest = max(time_to_hours(event.end) for event in events)

    earliest = ceil(earliest)
    latest = ceil(latest)

    ax.set_ylim(latest, earliest)
    ax.set_yticks(range(earliest, latest))
    ax.set_yticklabels([f"{h}:00" for h in range(earliest, latest)])

    ax.grid(axis="y", linestyle="--", linewidth=0.5)

    if save_img:
        plt.savefig(
            out_path,
            dpi=200,
            bbox_inches="tight",
        )

    if show:
        plt.show()


def time_to_hours(time: datetime.time) -> float:
    return time.hour + time.minute / 60



from pathlib import Path

import typer

from my_weekly_schedule.models import WeekdayEnum
from my_weekly_schedule.parse_txt import parse_txt
from my_weekly_schedule.plot_schedule import plot_events

app = typer.Typer()


@app.command()
def main(
    input_: Path,
    show: bool = True,
    save_img: bool = True,
    weekends: bool = True,
):
    """
    Plot weekly schedule schedule.
    """
    events = parse_txt(input_)

    if not weekends and any(
        event.day.value > WeekdayEnum.fri.value for event in events
    ):
        typer.echo("Attempted to plot without weekends, but there are weekend events.")
        raise typer.Abort()

    out_path = input_.with_suffix(".png")

    plot_events(
        events,
        with_weekends=weekends,
        show=show,
        save_img=save_img,
        out_path=out_path,
    )


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['my_weekly_schedule']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.5.1,<4.0.0', 'pydantic>=1.9.0,<2.0.0', 'typer>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['my_weekly_schedule = my_weekly_schedule.main:app']}

setup_kwargs = {
    'name': 'my-weekly-schedule',
    'version': '0.1.2',
    'description': 'A tool for plotting your weekly schedule.',
    'long_description': '# my_weekly_schedule\n\nPlot your weekly schedule from a simple text file with Python.\n\nInspired by [utkuufuk/weekplot](https://github.com/utkuufuk/weekplot).\n\n![Schedule](example.png)\n\n## Installation\n\nRun `pip install my_weekly_schedule` for the latest stable version.\n\n### Development\n\nThis project uses [Python Poetry](https://python-poetry.org/) for dependency management.\nYou can install Python Poetry with the [following instructions](https://python-poetry.org/docs/#installation).\n\n``` sh\npoetry install\n```\n\nTo run the code during development:\n\n``` sh\npoetry run python my_weekly_schedule/main.py example.txt\n```\n\n## Usage\n\nYou can run the default options with:\n\n```sh\nmy_weekly_schedule example.txt\n```\n\nYou can replace `example.txt` with your own schedule files.\n\nYou can list all CLI options with:\n\n```sh\nmy_weekly_schedule --help\n```\n\n#### Example input files\n - [text](example.txt)\n\n## Colors\n\nYou can use any color in [the CSS3 specification](https://www.w3.org/TR/css-color-3/#svg-color).\n',
    'author': 'Fabian Torres',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ftorres16/my_weekly_schedule',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
